'LCDFormat_002.bas
'Version    0.02
'Date       2022-06-14
'Purpose    Insert display of a real number right-aligned into a LCD display mask.
'           The conversion of a real number into a string for display is left-aligned.
'           However, the display should be right-aligned
'           with a fixed position of the decimal point.
'           The digits are inserted from right to left in the display mask.
'           In this case from LCD position 9 to 15, i.e. length 7 characters.
'           If the real number is longer than 7 characters, overflow '####' is shown
'           'Format' is used for string conversion.
'           Hex-Code is 116 bytes smaller than 'Fusing' conversion
'Author     Georg Latzel, DL6GL
'Copyright  (c) Georg Latzel, free for private use under GNU license
'Hardware   ATmega328P or Arduino or any AVR
'           16x2 LCD
'           LCD RS                        PB.2
'           LCD E                         PB.1
'           LCD D4                        PB.0
'           LCD D5                        PD.7
'           LCD D6                        PD.6
'           LCD D7                        PD.5
'
'Compiler   BASCOM-AVR 2.0.8.3
'           For readable "camel case" variables mark "Don't change case"
'           in BASCOM Options-Programmer-Environment-Editor
'
'ATmega FUSE-Bits (AVR Studio 4.0-Settings, [ ]=no check, [x]=check)
'In ATMEGA-Data sheets: Bit = 1 -> not set
'                       Bit = 0 -> set
'BODLEVEL   Brown-out detection disabled
'RSTDISBL   [ ]
'DWEN       [ ]
'SPIEN      [x] Do not change!
'WDTON      [ ]
'EESAVE     [ ]
'BOOTSZ     Boot Flash size=256 (Minimal setting)
'CKDIV8     [ ]
'CKOUT      [ ]
'SUT_CKSEL  Ext. Crystal Osc.8.0- MHz, Start-up time 16CK/14ms
'Extended:  &HFF
'High:      &HDF
'Low:       &HFF
'
'Edit history
'V0.02      2022-06-14, first release
'           Flash used: 2,124 bytes (6%)

'
$regfile = "m328pdef.dat"                         'ATMega328p
$crystal = 16000000                               'Xtal 16.000MHz
$hwstack = 40
$swstack = 40
$framesize = 40

'Configure LCD on Port D and B ------------------------------------------------
Config Lcd = 16 * 2
Config Lcdpin = Pin , Db7 = PortD.5 , Db6 = PortD.6 , Db5 = PortD.7
Config Lcdpin = Pin , Db4 = PortB.0 , E = PortB.1 , Rs = PortB.2


Dim bytLen As Byte
Dim bytRun As Byte
Dim bytTmp0 As Byte
Dim bytTmp1 As Byte
Dim dwdTmp0 As DWord
Dim sngTest As Single
Dim sngTmp0 As Single
Dim strSav As String * 16
Dim strNum As String * 16
Dim bytNum(16) As Byte At strNum Overlay
Dim strDis As String * 16                         'Text to be dispayed on LCD
Dim bytDis(16) As Byte At strDis Overlay          'the same as byte array

Declare Sub DisplayReal(byVal bytLenMax As Byte , byVal bytPosLast As Byte)


Cls
Cursor Off Noblink
Locate 1 , 1
LCD "LCD Format V0.02"

sngTest = 1234.567
sngTmp0 = sngTest
bytRun = 0
strDis = "Voltage        V"                       'Display mask
strSav = Space(16)

Do
   Incr bytRun
   'Use 'Format' for string conversion
   dwdTmp0 = sngTmp0 * 1000                       'Factor for 3 decimals
   strNum = str(dwdTmp0)
   strNum = Format(strNum , "0.000")
   If strNum <> strSav Then                     'Display if number was changed
      Call DisplayReal(7 , 15)
      Locate 2 , 1
      LCD strDis
      strSav = strNum
      Wait 2
   End If

   If bytRun = 5 Then
      bytRun = 0
      sngTmp0 = sngTest
   Else
      sngTmp0 = 0.1 * sngTmp0
   End If
Loop

'******************************************************************************
Sub DisplayReal(byVal bytLenMax As Byte , byVal bytPosLast As Byte)
'Insert temporary string  bytNum to display string bytDis
'from position bytPosLast backwards, max. bytLenMax characters
'Input:  bytNum    (=strNum by overlay) string to be inserted in strDis
'        bytLenMax   Max. possible length of string strNum to be inserted
'        bytPosLast  Right position in bytDis (=strDis by overlay)
'Output: strDis      Completed string to be displayed on LCD

         bytTmp1 = bytPosLast - bytLenMax
         bytTmp1 = bytTmp1 + 1
         bytLen = Len(strNum)
         If bytLen > bytLenMax Then               'No room, overflow
            For bytTmp0 = bytTmp1 To bytPosLast
               bytDis(bytTmp0) = "#"
            Next bytTmp0
            Exit Sub
         Else                                     'Fits available space
            For bytTmp0 = bytTmp1 To bytPosLast
               bytDis(bytTmp0) = " "              'Fill with blanks
            Next bytTmp0
         End If

         bytTmp1 = bytPosLast + 1
         For bytTmp0 = bytLen To 1 Step -1        'From right to left
             bytTmp1 = bytTmp1 - 1
             bytDis(bytTmp1) = bytNum(bytTmp0)
         Next bytTmp0

End Sub


End