'LCDFormat_003.bas
'Version    0.03
'Date       2022-06-15
'Purpose    Insert display of a integer number right-aligned into a LCD display mask.
'           The conversion of a number into a string for display is left-aligned.
'           However, the display should be right-aligned with a fixed end position.
'           The digits are inserted from right to left in the display mask.
'           In this case from LCD position 4 to 14, i.e. length 11 characters.
'           If the number is longer than 11 characters, overflow '####' is shown
'           'Format' is used for string conversion.
'Author     Georg Latzel, DL6GL
'Copyright  (c) Georg Latzel, free for private use under GNU license
'Hardware   ATmega328P or Arduino or any AVR
'           16x2 LCD
'           LCD RS                        PB.2
'           LCD E                         PB.1
'           LCD D4                        PB.0
'           LCD D5                        PD.7
'           LCD D6                        PD.6
'           LCD D7                        PD.5
'
'Compiler   BASCOM-AVR 2.0.8.3
'           For readable "camel case" variables mark "Don't change case"
'           in BASCOM Options-Programmer-Environment-Editor
'
'ATmega FUSE-Bits (AVR Studio 4.0-Settings, [ ]=no check, [x]=check)
'In ATMEGA-Data sheets: Bit = 1 -> not set
'                       Bit = 0 -> set
'BODLEVEL   Brown-out detection disabled
'RSTDISBL   [ ]
'DWEN       [ ]
'SPIEN      [x] Do not change!
'WDTON      [ ]
'EESAVE     [ ]
'BOOTSZ     Boot Flash size=256 (Minimal setting)
'CKDIV8     [ ]
'CKOUT      [ ]
'SUT_CKSEL  Ext. Crystal Osc.8.0- MHz, Start-up time 16CK/14ms
'Extended:  &HFF
'High:      &HDF
'Low:       &HFF
'
'Edit history
'V0.03      2022-06-15, first release
'           Flash used: 2,064 bytes (6%)

'
$regfile = "m328pdef.dat"                         'ATMega328p
$crystal = 16000000                               'Xtal 16.000MHz
$hwstack = 40
$swstack = 40
$framesize = 40

'Configure LCD on Port D and B ------------------------------------------------
Config Lcd = 16 * 2
Config Lcdpin = Pin , Db7 = PortD.5 , Db6 = PortD.6 , Db5 = PortD.7
Config Lcdpin = Pin , Db4 = PortB.0 , E = PortB.1 , Rs = PortB.2


Dim bytLen As Byte
Dim bytRun As Byte
Dim bytTmp0 As Byte
Dim bytTmp1 As Byte
Dim bytTmp2 As Byte
Dim bytDigit As Byte
Dim bytLCDStart As Byte
Dim bytLCDEnd As Byte
Dim bytLCDLen As Byte
Dim bytNumMax As Byte
Dim bytSep As Byte
Dim dwdTmp0 As DWord
Dim dwdTest As DWord
Dim strSav As String * 16
Dim strNum As String * 16
Dim bytNum(16) As Byte At strNum Overlay
Dim strDis As String * 16                         'Text to be dispayed on LCD
Dim bytDis(16) As Byte At strDis Overlay          'the same as byte array

Declare Sub DisplayInt(byVal bytLenMax As Byte , byVal bytPosLast As Byte , byVal bytSep As Byte)


Cls
Cursor Off Noblink
Locate 1 , 1
LCD "LCD Format V0.03"

dwdTest = 1234567890                              'Start frequency
dwdTmp0 = dwdTest
bytNumMax = 9                                     'Max number length excl. separators
bytLCDStart = 4                                   'Start position of frequency
bytLCDEnd = 14                                    'End position of frequency
bytLCDLen = bytLCDEnd - bytLCDStart
bytLCDLen = bytLCDLen + 1                         'Length of frequency string
bytSep = ","                                      'Thousand separator ','
'bytSep = "."                                      'Thousand separator '.'
'bytSep = "-"                                      'No Thousand separator
bytRun = 0
strDis = "f             Hz"                       'Display mask, thousand separator ','
strSav = Space(16)

Do
   Incr bytRun
   strNum = str(dwdTmp0)
   strNum = Format(strNum , "0")                  'Use 'Format' for string conversion
   If strNum <> strSav Then                       'Display if number was changed
      Call DisplayInt(bytLCDLen , bytLCDEnd , bytSep)
      Locate 2 , 1
      LCD strDis
      strSav = strNum
      Wait 2
   End If

   If bytRun = 10 Then
      bytRun = 0
      dwdTmp0 = dwdTest
   Else
      dwdTmp0 = dwdTmp0 / 10
   End If
Loop

'******************************************************************************
Sub DisplayInt(byVal bytLenMax As Byte , byVal bytPosLast As Byte , byVal bytSep As Byte)
'Insert integer number string  bytNum to display string bytDis
'from position bytPosLast backwards, max. bytLenMax characters
'Thousand separators are added.
'Input:  bytNum      (=strNum by overlay) string to be inserted in strDis
'        bytNumMax   Max. digits exclusive thousand separator
'        bytLenMax   Max. possible length of string to be inserted
'        bytPosLast  Right position in bytDis (=strDis by overlay)
'        bytSep      Thousand separator "," or "."
'                    = "-": No separator
'Output: strDis      Completed string to be displayed on LCD

   bytLen = Len(strNum)
   If bytLen > bytNumMax Then                     'No room, overflow
      For bytTmp0 = bytLCDStart To bytLCDEnd
          bytDis(bytTmp0) = "#"
      Next bytTmp0
      Exit Sub
   Else
      For bytTmp0 = bytLCDStart To bytLCDEnd
         bytDis(bytTmp0) = " "
      Next bytTmp0
   End If
   bytTmp1 = bytLCDEnd + 1
   bytDigit = 0
   For bytTmp0 = bytLen To 1 Step -1              'From right to left
       bytDigit = bytDigit + 1                    'Current digit number from right
       bytTmp1 = bytTmp1 - 1
       bytTmp2 = bytDigit Mod 3
       If bytTmp2 = 0 Then                        'Next positon is thousand separator
          bytDis(bytTmp1) = bytNum(bytTmp0)
          If bytTmp0 > 1 And bytSep <> "-" Then   'Add separator
            bytTmp1 = bytTmp1 - 1
            bytDis(bytTmp1) = bytSep
          End If
       Else
         bytDis(bytTmp1) = bytNum(bytTmp0)
       End If
   Next bytTmp0

End Sub

End